//
//  Copyright (c) 2022 Open Whisper Systems. All rights reserved.
//

import Foundation

// TODO define actual type, and validate length
public typealias IdentityKey = Data

public enum AccountServiceClientError: Error {
    case captchaRequired
}

/// based on libsignal-service-java's AccountManager class
@objc
public class AccountServiceClient: NSObject {

    // MARK: - Public

    public func requestPreauthChallenge(e164: String, pushToken: String, isVoipToken: Bool) -> Promise<Void> {
        serviceClient.requestPreauthChallenge(e164: e164, pushToken: pushToken, isVoipToken: isVoipToken)
    }

    public func requestVerificationCode(e164: String, preauthChallenge: String?, captchaToken: String?, transport: TSVerificationTransport) -> Promise<Void> {
        serviceClient.requestVerificationCode(e164: e164,
                                              preauthChallenge: preauthChallenge,
                                              captchaToken: captchaToken,
                                              transport: transport).recover { error -> Void in
            if error.httpStatusCode == 402 {
                throw AccountServiceClientError.captchaRequired
            }
            throw error
        }
    }

    public func getPreKeysCount(for identity: OWSIdentity) -> Promise<Int> {
        return serviceClient.getAvailablePreKeys(for: identity)
    }

    public func setPreKeys(for identity: OWSIdentity,
                           identityKey: IdentityKey,
                           signedPreKeyRecord: SignedPreKeyRecord,
                           preKeyRecords: [PreKeyRecord]) -> Promise<Void> {
        return serviceClient.registerPreKeys(for: identity,
                                             identityKey: identityKey,
                                             signedPreKeyRecord: signedPreKeyRecord,
                                             preKeyRecords: preKeyRecords)
    }

    public func setSignedPreKey(_ signedPreKey: SignedPreKeyRecord, for identity: OWSIdentity) -> Promise<Void> {
        return serviceClient.setCurrentSignedPreKey(signedPreKey, for: identity)
    }

    public func updatePrimaryDeviceAccountAttributes() -> Promise<Void> {
        return serviceClient.updatePrimaryDeviceAccountAttributes()
    }

    public func getAccountWhoAmI() -> Promise<WhoAmIResponse> {
        serviceClient.getAccountWhoAmI()
    }

    public func verifySecondaryDevice(verificationCode: String,
                                      phoneNumber: String,
                                      authKey: String,
                                      encryptedDeviceName: Data) -> Promise<VerifySecondaryDeviceResponse> {
        return serviceClient.verifySecondaryDevice(verificationCode: verificationCode, phoneNumber: phoneNumber, authKey: authKey, encryptedDeviceName: encryptedDeviceName)
    }
}
